// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <memory>

#include <aoc/Program.h>
#include <aoc/SolverEngine.h>

// Solver implementations in day order.
#include <aoc/HistorianHysteria.h>
#include <aoc/RedNosedReports.h>
#include <aoc/MullItOver.h>
#include <aoc/CeresSearch.h>
#include <aoc/PrintQueue.h>
#include <aoc/GuardGallivant.h>

void Program::run()
{
    std::cout << "### Advent of Code 2024 ###\n";
    runSolvers();
}

template <class T>
void runSolver(SolverEngine& solverEngine)
{
    auto solver = std::make_unique<T>();
    solverEngine.run(*solver);
}

void Program::runSolvers()
{
    SolverEngine solverEngine{ getInputPaths() };
    runSolver<HistorianHysteria>(solverEngine);
    runSolver<RedNosedReports>(solverEngine);
    runSolver<MullItOver>(solverEngine);
    runSolver<CeresSearch>(solverEngine);
    runSolver<PrintQueue>(solverEngine);
    runSolver<GuardGallivant>(solverEngine);
}

std::vector<std::string> Program::getInputPaths() const
{
    return std::vector<std::string>{ "data", "../../../data", "../../../../data" };
}
