// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <catch2/catch_test_macros.hpp>

#include <aoc/Solver.h>
#include <aoc/SolverEngine.h>
#include <aoc/HistorianHysteria.h>

#include <aocTests/Part1TestContext.h>
#include <aocTests/Part2TestContext.h>

#define REQUIRE_MESSAGE(cond, msg) if (!(cond)) { INFO(msg); REQUIRE(cond); }

class TestCaseBase
{
public:
    Part1TestContext part1TestContext{ getInputPaths() };
    Part2TestContext part2TestContext{ getInputPaths() };
    Part1TestContext part1ExampleTestContext{ getExampleInputPaths() };
    Part2TestContext part2ExampleTestContext{ getExampleInputPaths() };
private:
    std::vector<std::string> getInputPaths() const
    {
        return std::vector<std::string>{ "data", "../../../data", "../../../../data" };
    }
    std::vector<std::string> getExampleInputPaths() const
    {
        return std::vector<std::string>{ "data/examples", "../../../data/examples",
            "../../../../data/examples" };
    }
};

template <class T>
void runTest(const long long int expected, const TestContext& context)
{
    auto solver = std::make_unique<T>();
    SolverEngine solverEngine{ context.getInputPaths() };
    solverEngine.run(*solver);

    REQUIRE(expected == context.getResult(*solver));
}

TEST_CASE("[HistorianHysteriaTests]")
{
    TestCaseBase test;
    SECTION("FullData1")
    {
        runTest<HistorianHysteria>(2176849, test.part1TestContext);
    }
    SECTION("FullData2")
    {
        runTest<HistorianHysteria>(23384288, test.part2TestContext);
    }
    SECTION("ExampleData1")
    {
        runTest<HistorianHysteria>(11, test.part1ExampleTestContext);
    }
    SECTION("ExampleData2")
    {
        runTest<HistorianHysteria>(31, test.part2ExampleTestContext);
    }
}
